@@ -33,6 +33,7 @@ from allauth.headless.base.response import (
     ForbiddenResponse,
 )
 from allauth.headless.base.views import APIView, AuthenticatedAPIView
+from allauth.headless.internal import authkit
 from allauth.headless.internal.restkit.response import ErrorResponse
 
 
@@ -50,15 +51,17 @@ class ConfirmLoginCodeView(APIView):
     input_class = ConfirmLoginCodeInput
 
     def dispatch(self, request, *args, **kwargs):
+        auth_status = authkit.AuthenticationStatus(request)
+        self.stage = auth_status.get_pending_stage()
+        if not self.stage:
+            return ConflictResponse(request)
         self.user, self.pending_login = flows.login_by_code.get_pending_login(
-            request, peek=True
+            self.stage.login, peek=True
         )
         return super().dispatch(request, *args, **kwargs)
 
     def post(self, request, *args, **kwargs):
-        flows.login_by_code.perform_login_by_code(
-            self.request, self.user, None, self.pending_login
-        )
+        flows.login_by_code.perform_login_by_code(self.request, self.stage, None)
         return AuthenticationResponse(request)
 
     def get_input_kwargs(self):
@@ -68,6 +71,10 @@ class ConfirmLoginCodeView(APIView):
         )
         return kwargs
 
+    def handle_invalid_input(self, input):
+        flows.login_by_code.record_invalid_attempt(self.request, self.stage.login)
+        return super().handle_invalid_input(input)
+
 
 @method_decorator(rate_limit(action="login"), name="handle")
 class LoginView(APIView):
